@@ -1,4 +1,4 @@
-/**
+/*
  * ALAC audio encoder
  * Copyright (c) 2008  Jaikrishnan Menon <realityman@gmx.net>
  *
@@ -22,6 +22,7 @@
 #include "avcodec.h"
 #include "put_bits.h"
 #include "dsputil.h"
+#include "internal.h"
 #include "lpc.h"
 #include "mathops.h"
 
@@ -58,6 +59,8 @@ typedef struct AlacLPCContext {
 } AlacLPCContext;
 
 typedef struct AlacEncodeContext {
+    int frame_size;                     /**< current frame size               */
+    int verbatim;                       /**< current frame verbatim mode flag */
     int compression_level;
     int min_prediction_order;
     int max_prediction_order;
@@ -75,17 +78,15 @@ typedef struct AlacEncodeContext {
 } AlacEncodeContext;
 
 
-static void init_sample_buffers(AlacEncodeContext *s,
-                                const int16_t *input_samples)
+static void init_sample_buffers(AlacEncodeContext *s, int16_t **input_samples)
 {
     int ch, i;
 
     for (ch = 0; ch < s->avctx->channels; ch++) {
-        const int16_t *sptr = input_samples + ch;
-        for (i = 0; i < s->avctx->frame_size; i++) {
-            s->sample_buf[ch][i] = *sptr;
-            sptr += s->avctx->channels;
-        }
+        int32_t       *bptr = s->sample_buf[ch];
+        const int16_t *sptr = input_samples[ch];
+        for (i = 0; i < s->frame_size; i++)
+            bptr[i] = sptr[i];
     }
 }
 
@@ -117,14 +118,20 @@ static void encode_scalar(AlacEncodeContext *s, int x,
     }
 }
 
-static void write_frame_header(AlacEncodeContext *s, int is_verbatim)
+static void write_frame_header(AlacEncodeContext *s)
 {
-    put_bits(&s->pbctx, 3,  s->avctx->channels-1);          // No. of channels -1
-    put_bits(&s->pbctx, 16, 0);                             // Seems to be zero
-    put_bits(&s->pbctx, 1,  1);                             // Sample count is in the header
-    put_bits(&s->pbctx, 2,  0);                             // FIXME: Wasted bytes field
-    put_bits(&s->pbctx, 1,  is_verbatim);                   // Audio block is verbatim
-    put_bits32(&s->pbctx, s->avctx->frame_size);            // No. of samples in the frame
+    int encode_fs = 0;
+
+    if (s->frame_size < DEFAULT_FRAME_SIZE)
+        encode_fs = 1;
+
+    put_bits(&s->pbctx, 3,  s->avctx->channels-1);  // No. of channels -1
+    put_bits(&s->pbctx, 16, 0);                     // Seems to be zero
+    put_bits(&s->pbctx, 1,  encode_fs);             // Sample count is in the header
+    put_bits(&s->pbctx, 2,  0);                     // FIXME: Wasted bytes field
+    put_bits(&s->pbctx, 1,  s->verbatim);           // Audio block is verbatim
+    if (encode_fs)
+        put_bits32(&s->pbctx, s->frame_size);       // No. of samples in the frame
 }
 
 static void calc_predictor_params(AlacEncodeContext *s, int ch)
@@ -144,7 +151,7 @@ static void calc_predictor_params(AlacEncodeContext *s, int ch)
         s->lpc[ch].lpc_coeff[5] =  -25;
     } else {
         opt_order = ff_lpc_calc_coefs(&s->lpc_ctx, s->sample_buf[ch],
-                                      s->avctx->frame_size,
+                                      s->frame_size,
                                       s->min_prediction_order,
                                       s->max_prediction_order,
                                       ALAC_MAX_LPC_PRECISION, coefs, shift,
@@ -167,8 +174,8 @@ static int estimate_stereo_mode(int32_t *left_ch, int32_t *right_ch, int n)
     /* calculate sum of 2nd order residual for each channel */
     sum[0] = sum[1] = sum[2] = sum[3] = 0;
     for (i = 2; i < n; i++) {
-        lt = left_ch[i] - 2*left_ch[i-1] + left_ch[i-2];
-        rt = right_ch[i] - 2*right_ch[i-1] + right_ch[i-2];
+        lt =  left_ch[i] - 2 *  left_ch[i - 1] +  left_ch[i - 2];
+        rt = right_ch[i] - 2 * right_ch[i - 1] + right_ch[i - 2];
         sum[2] += FFABS((lt + rt) >> 1);
         sum[3] += FFABS(lt - rt);
         sum[0] += FFABS(lt);
@@ -184,9 +191,8 @@ static int estimate_stereo_mode(int32_t *left_ch, int32_t *right_ch, int n)
     /* return mode with lowest score */
     best = 0;
     for (i = 1; i < 4; i++) {
-        if (score[i] < score[best]) {
+        if (score[i] < score[best])
             best = i;
-        }
     }
     return best;
 }
@@ -194,45 +200,40 @@ static int estimate_stereo_mode(int32_t *left_ch, int32_t *right_ch, int n)
 static void alac_stereo_decorrelation(AlacEncodeContext *s)
 {
     int32_t *left = s->sample_buf[0], *right = s->sample_buf[1];
-    int i, mode, n = s->avctx->frame_size;
+    int i, mode, n = s->frame_size;
     int32_t tmp;
 
     mode = estimate_stereo_mode(left, right, n);
 
-    switch(mode)
-    {
-        case ALAC_CHMODE_LEFT_RIGHT:
-            s->interlacing_leftweight = 0;
-            s->interlacing_shift = 0;
-            break;
-
-        case ALAC_CHMODE_LEFT_SIDE:
-            for (i = 0; i < n; i++) {
-                right[i] = left[i] - right[i];
-            }
-            s->interlacing_leftweight = 1;
-            s->interlacing_shift = 0;
-            break;
-
-        case ALAC_CHMODE_RIGHT_SIDE:
-            for (i = 0; i < n; i++) {
-                tmp = right[i];
-                right[i] = left[i] - right[i];
-                left[i] = tmp + (right[i] >> 31);
-            }
-            s->interlacing_leftweight = 1;
-            s->interlacing_shift = 31;
-            break;
-
-        default:
-            for (i = 0; i < n; i++) {
-                tmp = left[i];
-                left[i] = (tmp + right[i]) >> 1;
-                right[i] = tmp - right[i];
-            }
-            s->interlacing_leftweight = 1;
-            s->interlacing_shift = 1;
-            break;
+    switch (mode) {
+    case ALAC_CHMODE_LEFT_RIGHT:
+        s->interlacing_leftweight = 0;
+        s->interlacing_shift      = 0;
+        break;
+    case ALAC_CHMODE_LEFT_SIDE:
+        for (i = 0; i < n; i++)
+            right[i] = left[i] - right[i];
+        s->interlacing_leftweight = 1;
+        s->interlacing_shift      = 0;
+        break;
+    case ALAC_CHMODE_RIGHT_SIDE:
+        for (i = 0; i < n; i++) {
+            tmp = right[i];
+            right[i] = left[i] - right[i];
+            left[i]  = tmp + (right[i] >> 31);
+        }
+        s->interlacing_leftweight = 1;
+        s->interlacing_shift      = 31;
+        break;
+    default:
+        for (i = 0; i < n; i++) {
+            tmp = left[i];
+            left[i]  = (tmp + right[i]) >> 1;
+            right[i] =  tmp - right[i];
+        }
+        s->interlacing_leftweight = 1;
+        s->interlacing_shift      = 1;
+        break;
     }
 }
 
@@ -244,8 +245,10 @@ static void alac_linear_predictor(AlacEncodeContext *s, int ch)
     if (lpc.lpc_order == 31) {
         s->predictor_buf[0] = s->sample_buf[ch][0];
 
-        for (i = 1; i < s->avctx->frame_size; i++)
-            s->predictor_buf[i] = s->sample_buf[ch][i] - s->sample_buf[ch][i-1];
+        for (i = 1; i < s->frame_size; i++) {
+            s->predictor_buf[i] = s->sample_buf[ch][i    ] -
+                                  s->sample_buf[ch][i - 1];
+        }
 
         return;
     }
@@ -262,12 +265,12 @@ static void alac_linear_predictor(AlacEncodeContext *s, int ch)
             residual[i] = sign_extend(samples[i] - samples[i-1], s->write_sample_size);
 
         // perform lpc on remaining samples
-        for (i = lpc.lpc_order + 1; i < s->avctx->frame_size; i++) {
+        for (i = lpc.lpc_order + 1; i < s->frame_size; i++) {
             int sum = 1 << (lpc.lpc_quant - 1), res_val, j;
 
             for (j = 0; j < lpc.lpc_order; j++) {
                 sum += (samples[lpc.lpc_order-j] - samples[0]) *
-                        lpc.lpc_coeff[j];
+                       lpc.lpc_coeff[j];
             }
 
             sum >>= lpc.lpc_quant;
@@ -276,21 +279,20 @@ static void alac_linear_predictor(AlacEncodeContext *s, int ch)
                                       s->write_sample_size);
             res_val = residual[i];
 
-            if(res_val) {
+            if (res_val) {
                 int index = lpc.lpc_order - 1;
                 int neg = (res_val < 0);
 
-                while(index >= 0 && (neg ? (res_val < 0):(res_val > 0))) {
-                    int val = samples[0] - samples[lpc.lpc_order - index];
+                while (index >= 0 && (neg ? (res_val < 0) : (res_val > 0))) {
+                    int val  = samples[0] - samples[lpc.lpc_order - index];
                     int sign = (val ? FFSIGN(val) : 0);
 
-                    if(neg)
-                        sign*=-1;
+                    if (neg)
+                        sign *= -1;
 
                     lpc.lpc_coeff[index] -= sign;
                     val *= sign;
-                    res_val -= ((val >> lpc.lpc_quant) *
-                            (lpc.lpc_order - index));
+                    res_val -= (val >> lpc.lpc_quant) * (lpc.lpc_order - index);
                     index--;
                 }
             }
@@ -305,100 +307,123 @@ static void alac_entropy_coder(AlacEncodeContext *s)
     int sign_modifier = 0, i, k;
     int32_t *samples = s->predictor_buf;
 
-    for (i = 0; i < s->avctx->frame_size;) {
+    for (i = 0; i < s->frame_size;) {
         int x;
 
         k = av_log2((history >> 9) + 3);
 
-        x = -2*(*samples)-1;
-        x ^= (x>>31);
+        x  = -2 * (*samples) -1;
+        x ^= x >> 31;
 
         samples++;
         i++;
 
         encode_scalar(s, x - sign_modifier, k, s->write_sample_size);
 
-        history += x * s->rc.history_mult
-                   - ((history * s->rc.history_mult) >> 9);
+        history += x * s->rc.history_mult -
+                   ((history * s->rc.history_mult) >> 9);
 
         sign_modifier = 0;
         if (x > 0xFFFF)
             history = 0xFFFF;
 
-        if (history < 128 && i < s->avctx->frame_size) {
+        if (history < 128 && i < s->frame_size) {
             unsigned int block_size = 0;
 
             k = 7 - av_log2(history) + ((history + 16) >> 6);
 
-            while (*samples == 0 && i < s->avctx->frame_size) {
+            while (*samples == 0 && i < s->frame_size) {
                 samples++;
                 i++;
                 block_size++;
             }
             encode_scalar(s, block_size, k, 16);
-
             sign_modifier = (block_size <= 0xFFFF);
-
             history = 0;
         }
 
     }
 }
 
-static void write_compressed_frame(AlacEncodeContext *s)
+static int write_frame(AlacEncodeContext *s, AVPacket *avpkt, int16_t **samples)
 {
     int i, j;
     int prediction_type = 0;
+    PutBitContext *pb = &s->pbctx;
 
-    if (s->avctx->channels == 2)
-        alac_stereo_decorrelation(s);
-    put_bits(&s->pbctx, 8, s->interlacing_shift);
-    put_bits(&s->pbctx, 8, s->interlacing_leftweight);
+    init_put_bits(pb, avpkt->data, avpkt->size);
 
-    for (i = 0; i < s->avctx->channels; i++) {
+    if (s->verbatim) {
+        write_frame_header(s);
+        /* samples are channel-interleaved in verbatim mode */
+        for (i = 0; i < s->frame_size; i++)
+            for (j = 0; j < s->avctx->channels; j++)
+                put_sbits(pb, 16, samples[j][i]);
+    } else {
+        init_sample_buffers(s, samples);
+        write_frame_header(s);
 
-        calc_predictor_params(s, i);
+        if (s->avctx->channels == 2)
+            alac_stereo_decorrelation(s);
+        put_bits(pb, 8, s->interlacing_shift);
+        put_bits(pb, 8, s->interlacing_leftweight);
 
-        put_bits(&s->pbctx, 4, prediction_type);
-        put_bits(&s->pbctx, 4, s->lpc[i].lpc_quant);
+        for (i = 0; i < s->avctx->channels; i++) {
+            calc_predictor_params(s, i);
 
-        put_bits(&s->pbctx, 3, s->rc.rice_modifier);
-        put_bits(&s->pbctx, 5, s->lpc[i].lpc_order);
-        // predictor coeff. table
-        for (j = 0; j < s->lpc[i].lpc_order; j++) {
-            put_sbits(&s->pbctx, 16, s->lpc[i].lpc_coeff[j]);
-        }
-    }
+            put_bits(pb, 4, prediction_type);
+            put_bits(pb, 4, s->lpc[i].lpc_quant);
 
-    // apply lpc and entropy coding to audio samples
-
-    for (i = 0; i < s->avctx->channels; i++) {
-        alac_linear_predictor(s, i);
-
-        // TODO: determine when this will actually help. for now it's not used.
-        if (prediction_type == 15) {
-            // 2nd pass 1st order filter
-            for (j = s->avctx->frame_size - 1; j > 0; j--)
-                s->predictor_buf[j] -= s->predictor_buf[j - 1];
+            put_bits(pb, 3, s->rc.rice_modifier);
+            put_bits(pb, 5, s->lpc[i].lpc_order);
+            // predictor coeff. table
+            for (j = 0; j < s->lpc[i].lpc_order; j++)
+                put_sbits(pb, 16, s->lpc[i].lpc_coeff[j]);
         }
 
-        alac_entropy_coder(s);
+        // apply lpc and entropy coding to audio samples
+
+        for (i = 0; i < s->avctx->channels; i++) {
+            alac_linear_predictor(s, i);
+
+            // TODO: determine when this will actually help. for now it's not used.
+            if (prediction_type == 15) {
+                // 2nd pass 1st order filter
+                for (j = s->frame_size - 1; j > 0; j--)
+                    s->predictor_buf[j] -= s->predictor_buf[j - 1];
+            }
+
+            alac_entropy_coder(s);
+        }
     }
+    put_bits(pb, 3, 7);
+    flush_put_bits(pb);
+    return put_bits_count(pb) >> 3;
+}
+
+static av_always_inline int get_max_frame_size(int frame_size, int ch, int bps)
+{
+    int header_bits = 23 + 32 * (frame_size < DEFAULT_FRAME_SIZE);
+    return FFALIGN(header_bits + bps * ch * frame_size + 3, 8) / 8;
+}
+
+static av_cold int alac_encode_close(AVCodecContext *avctx)
+{
+    AlacEncodeContext *s = avctx->priv_data;
+    ff_lpc_end(&s->lpc_ctx);
+    av_freep(&avctx->extradata);
+    avctx->extradata_size = 0;
+    av_freep(&avctx->coded_frame);
+    return 0;
 }
 
 static av_cold int alac_encode_init(AVCodecContext *avctx)
 {
-    AlacEncodeContext *s    = avctx->priv_data;
+    AlacEncodeContext *s = avctx->priv_data;
     int ret;
-    uint8_t *alac_extradata = av_mallocz(ALAC_EXTRADATA_SIZE+1);
+    uint8_t *alac_extradata;
 
-    avctx->frame_size      = DEFAULT_FRAME_SIZE;
-    avctx->bits_per_coded_sample = DEFAULT_SAMPLE_SIZE;
-
-    if (avctx->sample_fmt != AV_SAMPLE_FMT_S16) {
-        av_log(avctx, AV_LOG_ERROR, "only pcm_s16 input samples are supported\n");
-        return -1;
-    }
+    avctx->frame_size = s->frame_size = DEFAULT_FRAME_SIZE;
 
     /* TODO: Correctly implement multi-channel ALAC.
              It is similar to multi-channel AAC, in that it has a series of
@@ -420,18 +445,29 @@ static av_cold int alac_encode_init(AVCodecContext *avctx)
     s->rc.k_modifier      = 14;
     s->rc.rice_modifier   = 4;
 
-    s->max_coded_frame_size = 8 + (avctx->frame_size*avctx->channels*avctx->bits_per_coded_sample>>3);
+    s->max_coded_frame_size = get_max_frame_size(avctx->frame_size,
+                                                 avctx->channels,
+                                                 DEFAULT_SAMPLE_SIZE);
 
-    s->write_sample_size  = avctx->bits_per_coded_sample + avctx->channels - 1; // FIXME: consider wasted_bytes
+    // FIXME: consider wasted_bytes
+    s->write_sample_size  = DEFAULT_SAMPLE_SIZE + avctx->channels - 1;
 
+    avctx->extradata = av_mallocz(ALAC_EXTRADATA_SIZE + FF_INPUT_BUFFER_PADDING_SIZE);
+    if (!avctx->extradata) {
+        ret = AVERROR(ENOMEM);
+        goto error;
+    }
+    avctx->extradata_size = ALAC_EXTRADATA_SIZE;
+
+    alac_extradata = avctx->extradata;
     AV_WB32(alac_extradata,    ALAC_EXTRADATA_SIZE);
     AV_WB32(alac_extradata+4,  MKBETAG('a','l','a','c'));
     AV_WB32(alac_extradata+12, avctx->frame_size);
-    AV_WB8 (alac_extradata+17, avctx->bits_per_coded_sample);
+    AV_WB8 (alac_extradata+17, DEFAULT_SAMPLE_SIZE);
     AV_WB8 (alac_extradata+21, avctx->channels);
     AV_WB32(alac_extradata+24, s->max_coded_frame_size);
     AV_WB32(alac_extradata+28,
-            avctx->sample_rate * avctx->channels * avctx->bits_per_coded_sample); // average bitrate
+            avctx->sample_rate * avctx->channels * DEFAULT_SAMPLE_SIZE); // average bitrate
     AV_WB32(alac_extradata+32, avctx->sample_rate);
 
     // Set relevant extradata fields
@@ -447,7 +483,8 @@ static av_cold int alac_encode_init(AVCodecContext *avctx)
            avctx->min_prediction_order > ALAC_MAX_LPC_ORDER) {
             av_log(avctx, AV_LOG_ERROR, "invalid min prediction order: %d\n",
                    avctx->min_prediction_order);
-                return -1;
+            ret = AVERROR(EINVAL);
+            goto error;
         }
 
         s->min_prediction_order = avctx->min_prediction_order;
@@ -459,7 +496,8 @@ static av_cold int alac_encode_init(AVCodecContext *avctx)
             avctx->max_prediction_order > ALAC_MAX_LPC_ORDER) {
             av_log(avctx, AV_LOG_ERROR, "invalid max prediction order: %d\n",
                    avctx->max_prediction_order);
-                return -1;
+            ret = AVERROR(EINVAL);
+            goto error;
         }
 
         s->max_prediction_order = avctx->max_prediction_order;
@@ -469,93 +507,74 @@ static av_cold int alac_encode_init(AVCodecContext *avctx)
         av_log(avctx, AV_LOG_ERROR,
                "invalid prediction orders: min=%d max=%d\n",
                s->min_prediction_order, s->max_prediction_order);
-        return -1;
+        ret = AVERROR(EINVAL);
+        goto error;
     }
 
-    avctx->extradata = alac_extradata;
-    avctx->extradata_size = ALAC_EXTRADATA_SIZE;
-
     avctx->coded_frame = avcodec_alloc_frame();
-    avctx->coded_frame->key_frame = 1;
+    if (!avctx->coded_frame) {
+        ret = AVERROR(ENOMEM);
+        goto error;
+    }
 
     s->avctx = avctx;
-    ret = ff_lpc_init(&s->lpc_ctx, avctx->frame_size, s->max_prediction_order,
-                      FF_LPC_TYPE_LEVINSON);
 
+    if ((ret = ff_lpc_init(&s->lpc_ctx, avctx->frame_size,
+                           s->max_prediction_order,
+                           FF_LPC_TYPE_LEVINSON)) < 0) {
+        goto error;
+    }
+
+    return 0;
+error:
+    alac_encode_close(avctx);
     return ret;
 }
 
-static int alac_encode_frame(AVCodecContext *avctx, uint8_t *frame,
-                             int buf_size, void *data)
+static int alac_encode_frame(AVCodecContext *avctx, AVPacket *avpkt,
+                             const AVFrame *frame, int *got_packet_ptr)
 {
     AlacEncodeContext *s = avctx->priv_data;
-    PutBitContext *pb = &s->pbctx;
-    int i, out_bytes, verbatim_flag = 0;
+    int out_bytes, max_frame_size, ret;
+    int16_t **samples = (int16_t **)frame->extended_data;
 
-    if (avctx->frame_size > DEFAULT_FRAME_SIZE) {
-        av_log(avctx, AV_LOG_ERROR, "input frame size exceeded\n");
-        return -1;
-    }
+    s->frame_size = frame->nb_samples;
 
-    if (buf_size < 2 * s->max_coded_frame_size) {
-        av_log(avctx, AV_LOG_ERROR, "buffer size is too small\n");
-        return -1;
-    }
+    if (frame->nb_samples < DEFAULT_FRAME_SIZE)
+        max_frame_size = get_max_frame_size(s->frame_size, avctx->channels,
+                                            DEFAULT_SAMPLE_SIZE);
+    else
+        max_frame_size = s->max_coded_frame_size;
 
-verbatim:
-    init_put_bits(pb, frame, buf_size);
+    if ((ret = ff_alloc_packet2(avctx, avpkt, 2 * max_frame_size)))
+        return ret;
 
-    if (s->compression_level == 0 || verbatim_flag) {
-        // Verbatim mode
-        const int16_t *samples = data;
-        write_frame_header(s, 1);
-        for (i = 0; i < avctx->frame_size * avctx->channels; i++) {
-            put_sbits(pb, 16, *samples++);
-        }
-    } else {
-        init_sample_buffers(s, data);
-        write_frame_header(s, 0);
-        write_compressed_frame(s);
-    }
+    /* use verbatim mode for compression_level 0 */
+    s->verbatim = !s->compression_level;
 
-    put_bits(pb, 3, 7);
-    flush_put_bits(pb);
-    out_bytes = put_bits_count(pb) >> 3;
+    out_bytes = write_frame(s, avpkt, samples);
 
-    if (out_bytes > s->max_coded_frame_size) {
+    if (out_bytes > max_frame_size) {
         /* frame too large. use verbatim mode */
-        if (verbatim_flag || s->compression_level == 0) {
-            /* still too large. must be an error. */
-            av_log(avctx, AV_LOG_ERROR, "error encoding frame\n");
-            return -1;
-        }
-        verbatim_flag = 1;
-        goto verbatim;
+        s->verbatim = 1;
+        out_bytes = write_frame(s, avpkt, samples);
     }
 
-    return out_bytes;
-}
-
-static av_cold int alac_encode_close(AVCodecContext *avctx)
-{
-    AlacEncodeContext *s = avctx->priv_data;
-    ff_lpc_end(&s->lpc_ctx);
-    av_freep(&avctx->extradata);
-    avctx->extradata_size = 0;
-    av_freep(&avctx->coded_frame);
+    avpkt->size = out_bytes;
+    *got_packet_ptr = 1;
     return 0;
 }
 
 AVCodec ff_alac_encoder = {
     .name           = "alac",
     .type           = AVMEDIA_TYPE_AUDIO,
-    .id             = CODEC_ID_ALAC,
+    .id             = AV_CODEC_ID_ALAC,
     .priv_data_size = sizeof(AlacEncodeContext),
     .init           = alac_encode_init,
-    .encode         = alac_encode_frame,
+    .encode2        = alac_encode_frame,
     .close          = alac_encode_close,
-    .capabilities = CODEC_CAP_SMALL_LAST_FRAME,
-    .sample_fmts = (const enum AVSampleFormat[]){ AV_SAMPLE_FMT_S16,
-                                                  AV_SAMPLE_FMT_NONE },
-    .long_name = NULL_IF_CONFIG_SMALL("ALAC (Apple Lossless Audio Codec)"),
+    .capabilities   = CODEC_CAP_SMALL_LAST_FRAME,
+    .sample_fmts    = (const enum AVSampleFormat[]){ AV_SAMPLE_FMT_S16P,
+                                                     AV_SAMPLE_FMT_NONE },
+    .long_name      = NULL_IF_CONFIG_SMALL("ALAC (Apple Lossless Audio Codec)"),
 };
