@@ -20,7 +20,6 @@
  */
 
 #include "avcodec.h"
-#include "internal.h"
 #include "bytestream.h"
 
 /**
@@ -269,7 +268,7 @@ static void cdg_scroll(CDGraphicsContext *cc, uint8_t *data,
 static int cdg_decode_frame(AVCodecContext *avctx,
                             void *data, int *data_size, AVPacket *avpkt)
 {
-    GetByteContext gb;
+    const uint8_t *buf = avpkt->data;
     int buf_size       = avpkt->size;
     int ret;
     uint8_t command, inst;
@@ -286,19 +285,19 @@ static int cdg_decode_frame(AVCodecContext *avctx,
         return AVERROR(EINVAL);
     }
 
-    bytestream2_init(&gb, avpkt->data, avpkt->size);
-
     ret = avctx->reget_buffer(avctx, &cc->frame);
     if (ret) {
         av_log(avctx, AV_LOG_ERROR, "reget_buffer() failed\n");
         return ret;
     }
 
-    command = bytestream2_get_byte(&gb);
-    inst    = bytestream2_get_byte(&gb);
+    command = bytestream_get_byte(&buf);
+    inst    = bytestream_get_byte(&buf);
     inst    &= CDG_MASK;
-    bytestream2_skip(&gb, 2);
-    bytestream2_get_buffer(&gb, cdg_data, sizeof(cdg_data));
+    buf += 2;  /// skipping 2 unneeded bytes
+
+    if (buf_size > CDG_HEADER_SIZE)
+        bytestream_get_buffer(&buf, cdg_data, buf_size - CDG_HEADER_SIZE);
 
     if ((command & CDG_MASK) == CDG_COMMAND) {
         switch (inst) {
@@ -340,7 +339,7 @@ static int cdg_decode_frame(AVCodecContext *avctx,
             }
 
             cdg_init_frame(&new_frame);
-            ret = ff_get_buffer(avctx, &new_frame);
+            ret = avctx->get_buffer(avctx, &new_frame);
             if (ret) {
                 av_log(avctx, AV_LOG_ERROR, "get_buffer() failed\n");
                 return ret;
@@ -357,10 +356,11 @@ static int cdg_decode_frame(AVCodecContext *avctx,
         *data_size = sizeof(AVFrame);
     } else {
         *data_size = 0;
+        buf_size   = 0;
     }
 
     *(AVFrame *) data = cc->frame;
-    return avpkt->size;
+    return buf_size;
 }
 
 static av_cold int cdg_decode_end(AVCodecContext *avctx)
@@ -376,11 +376,11 @@ static av_cold int cdg_decode_end(AVCodecContext *avctx)
 AVCodec ff_cdgraphics_decoder = {
     .name           = "cdgraphics",
     .type           = AVMEDIA_TYPE_VIDEO,
-    .id             = CODEC_ID_CDGRAPHICS,
+    .id             = AV_CODEC_ID_CDGRAPHICS,
     .priv_data_size = sizeof(CDGraphicsContext),
     .init           = cdg_decode_init,
     .close          = cdg_decode_end,
     .decode         = cdg_decode_frame,
     .capabilities   = CODEC_CAP_DR1,
-    .long_name = NULL_IF_CONFIG_SMALL("CD Graphics video"),
+    .long_name      = NULL_IF_CONFIG_SMALL("CD Graphics video"),
 };
