@@ -518,7 +518,7 @@ static av_cold int qtrle_decode_end(AVCodecContext *avctx)
 AVCodec ff_qtrle_decoder = {
     .name           = "qtrle",
     .type           = AVMEDIA_TYPE_VIDEO,
-    .id             = CODEC_ID_QTRLE,
+    .id             = AV_CODEC_ID_QTRLE,
     .priv_data_size = sizeof(QtrleContext),
     .init           = qtrle_decode_init,
     .close          = qtrle_decode_end,
