@@ -43,10 +43,9 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "libavutil/common.h"
 #include "libavutil/intreadwrite.h"
 #include "avcodec.h"
-#include "internal.h"
-#include "bytestream.h"
 
 #define VMD_HEADER_SIZE 0x330
 #define PALETTE_COUNT 256
@@ -77,6 +76,8 @@ typedef struct VmdVideoContext {
 static void lz_unpack(const unsigned char *src, int src_len,
                       unsigned char *dest, int dest_len)
 {
+    const unsigned char *s;
+    const unsigned char *s_end;
     unsigned char *d;
     unsigned char *d_end;
     unsigned char queue[QUEUE_SIZE];
@@ -87,17 +88,19 @@ static void lz_unpack(const unsigned char *src, int src_len,
     unsigned int speclen;
     unsigned char tag;
     unsigned int i, j;
-    GetByteContext gb;
 
-    bytestream2_init(&gb, src, src_len);
+    s = src;
+    s_end = src + src_len;
     d = dest;
     d_end = d + dest_len;
-    dataleft = bytestream2_get_le32(&gb);
-    memset(queue, 0x20, QUEUE_SIZE);
-    if (bytestream2_get_bytes_left(&gb) < 4)
+
+    if (s_end - s < 8)
         return;
-    if (bytestream2_peek_le32(&gb) == 0x56781234) {
-        bytestream2_get_le32(&gb);
+    dataleft = AV_RL32(s);
+    s += 4;
+    memset(queue, 0x20, QUEUE_SIZE);
+    if (AV_RL32(s) == 0x56781234) {
+        s += 4;
         qpos = 0x111;
         speclen = 0xF + 3;
     } else {
@@ -105,13 +108,13 @@ static void lz_unpack(const unsigned char *src, int src_len,
         speclen = 100;  /* no speclen */
     }
 
-    while (dataleft > 0 && bytestream2_get_bytes_left(&gb) > 0) {
-        tag = bytestream2_get_byteu(&gb);
+    while (s_end - s > 0 && dataleft > 0) {
+        tag = *s++;
         if ((tag == 0xFF) && (dataleft > 8)) {
-            if (d_end - d < 8 || bytestream2_get_bytes_left(&gb) < 8)
+            if (d_end - d < 8 || s_end - s < 8)
                 return;
             for (i = 0; i < 8; i++) {
-                queue[qpos++] = *d++ = bytestream2_get_byteu(&gb);
+                queue[qpos++] = *d++ = *s++;
                 qpos &= QUEUE_MASK;
             }
             dataleft -= 8;
@@ -120,17 +123,21 @@ static void lz_unpack(const unsigned char *src, int src_len,
                 if (dataleft == 0)
                     break;
                 if (tag & 0x01) {
-                    if (d_end - d < 1 || bytestream2_get_bytes_left(&gb) < 1)
+                    if (d_end - d < 1 || s_end - s < 1)
                         return;
-                    queue[qpos++] = *d++ = bytestream2_get_byte(&gb);
+                    queue[qpos++] = *d++ = *s++;
                     qpos &= QUEUE_MASK;
                     dataleft--;
                 } else {
-                    chainofs = bytestream2_get_byte(&gb);
-                    chainofs |= ((bytestream2_peek_byte(&gb) & 0xF0) << 4);
-                    chainlen = (bytestream2_get_byte(&gb) & 0x0F) + 3;
+                    if (s_end - s < 2)
+                        return;
+                    chainofs = *s++;
+                    chainofs |= ((*s & 0xF0) << 4);
+                    chainlen = (*s++ & 0x0F) + 3;
                     if (chainlen == speclen) {
-                        chainlen = bytestream2_get_byte(&gb) + 0xF + 3;
+                        if (s_end - s < 1)
+                            return;
+                        chainlen = *s++ + 0xF + 3;
                     }
                     if (d_end - d < chainlen)
                         return;
@@ -146,47 +153,51 @@ static void lz_unpack(const unsigned char *src, int src_len,
         }
     }
 }
-static int rle_unpack(const unsigned char *src, unsigned char *dest,
-                      int src_count, int src_size, int dest_len)
-{
-    unsigned char *pd;
-    int i, l;
-    unsigned char *dest_end = dest + dest_len;
-    GetByteContext gb;
 
-    bytestream2_init(&gb, src, src_size);
+static int rle_unpack(const unsigned char *src, int src_len, int src_count,
+                      unsigned char *dest, int dest_len)
+{
+    const unsigned char *ps;
+    const unsigned char *ps_end;
+    unsigned char *pd;
+    int i, j, l;
+    unsigned char *dest_end = dest + dest_len;
+
+    ps = src;
+    ps_end = src + src_len;
     pd = dest;
     if (src_count & 1) {
-        if (bytestream2_get_bytes_left(&gb) < 1)
+        if (ps_end - ps < 1)
             return 0;
-        *pd++ = bytestream2_get_byteu(&gb);
+        *pd++ = *ps++;
     }
 
     src_count >>= 1;
     i = 0;
     do {
-        if (bytestream2_get_bytes_left(&gb) < 1)
+        if (ps_end - ps < 1)
             break;
-        l = bytestream2_get_byteu(&gb);
+        l = *ps++;
         if (l & 0x80) {
             l = (l & 0x7F) * 2;
-            if (dest_end - pd < l || bytestream2_get_bytes_left(&gb) < l)
-                return bytestream2_tell(&gb);
-            bytestream2_get_buffer(&gb, pd, l);
+            if (dest_end - pd < l || ps_end - ps < l)
+                return ps - src;
+            memcpy(pd, ps, l);
+            ps += l;
             pd += l;
         } else {
-            if (dest_end - pd < i || bytestream2_get_bytes_left(&gb) < 2)
-                return bytestream2_tell(&gb);
-            for (i = 0; i < l; i++) {
-                *pd++ = bytestream2_get_byteu(&gb);
-                *pd++ = bytestream2_get_byteu(&gb);
+            if (dest_end - pd < 2*l || ps_end - ps < 2)
+                return ps - src;
+            for (j = 0; j < l; j++) {
+                *pd++ = ps[0];
+                *pd++ = ps[1];
             }
-            bytestream2_skip(&gb, 2);
+            ps += 2;
         }
         i += l;
     } while (i < src_count);
 
-    return bytestream2_tell(&gb);
+    return ps - src;
 }
 
 static void vmd_decode(VmdVideoContext *s)
@@ -195,8 +206,12 @@ static void vmd_decode(VmdVideoContext *s)
     unsigned int *palette32;
     unsigned char r, g, b;
 
-    GetByteContext gb;
+    /* point to the start of the encoded data */
+    const unsigned char *p = s->buf + 16;
+    const unsigned char *p_end = s->buf + s->size;
 
+    const unsigned char *pb;
+    const unsigned char *pb_end;
     unsigned char meth;
     unsigned char *dp;   /* pointer to current frame */
     unsigned char *pp;   /* pointer to previous frame */
@@ -241,31 +256,29 @@ static void vmd_decode(VmdVideoContext *s)
     }
 
     /* check if there is a new palette */
-    bytestream2_init(&gb, s->buf + 16, s->size - 16);
     if (s->buf[15] & 0x02) {
-        bytestream2_skip(&gb, 2);
+        if (p_end - p < 2 + 3 * PALETTE_COUNT)
+            return;
+        p += 2;
         palette32 = (unsigned int *)s->palette;
-        if (bytestream2_get_bytes_left(&gb) >= PALETTE_COUNT * 3) {
-            for (i = 0; i < PALETTE_COUNT; i++) {
-                r = bytestream2_get_byteu(&gb) * 4;
-                g = bytestream2_get_byteu(&gb) * 4;
-                b = bytestream2_get_byteu(&gb) * 4;
-                palette32[i] = 0xFFU << 24 | (r << 16) | (g << 8) | (b);
-                palette32[i] |= palette32[i] >> 6 & 0x30303;
-            }
+        for (i = 0; i < PALETTE_COUNT; i++) {
+            r = *p++ * 4;
+            g = *p++ * 4;
+            b = *p++ * 4;
+            palette32[i] = 0xFF << 24 | r << 16 | g << 8 | b;
+            palette32[i] |= palette32[i] >> 6 & 0x30303;
         }
     }
-    if (s->size > 0) {
+    if (p < p_end) {
         /* originally UnpackFrame in VAG's code */
-        bytestream2_init(&gb, gb.buffer, s->buf + s->size - gb.buffer);
-        if (bytestream2_get_bytes_left(&gb) < 1)
-            return;
-        meth = bytestream2_get_byteu(&gb);
+        pb = p;
+        pb_end = p_end;
+        meth = *pb++;
         if (meth & 0x80) {
-            lz_unpack(gb.buffer, bytestream2_get_bytes_left(&gb),
-                      s->unpack_buffer, s->unpack_buffer_size);
+            lz_unpack(pb, p_end - pb, s->unpack_buffer, s->unpack_buffer_size);
             meth &= 0x7F;
-            bytestream2_init(&gb, s->unpack_buffer, s->unpack_buffer_size);
+            pb = s->unpack_buffer;
+            pb_end = s->unpack_buffer + s->unpack_buffer_size;
         }
 
         dp = &s->frame.data[0][frame_y * s->frame.linesize[0] + frame_x];
@@ -275,12 +288,15 @@ static void vmd_decode(VmdVideoContext *s)
             for (i = 0; i < frame_height; i++) {
                 ofs = 0;
                 do {
-                    len = bytestream2_get_byte(&gb);
+                    if (pb_end - pb < 1)
+                        return;
+                    len = *pb++;
                     if (len & 0x80) {
                         len = (len & 0x7F) + 1;
-                        if (ofs + len > frame_width || bytestream2_get_bytes_left(&gb) < len)
+                        if (ofs + len > frame_width || pb_end - pb < len)
                             return;
-                        bytestream2_get_buffer(&gb, &dp[ofs], len);
+                        memcpy(&dp[ofs], pb, len);
+                        pb += len;
                         ofs += len;
                     } else {
                         /* interframe pixel copy */
@@ -291,7 +307,7 @@ static void vmd_decode(VmdVideoContext *s)
                     }
                 } while (ofs < frame_width);
                 if (ofs > frame_width) {
-                    av_log(s->avctx, AV_LOG_ERROR, "VMD video: offset > width (%d > %d)\n",
+                    av_log(s->avctx, AV_LOG_ERROR, "offset > width (%d > %d)\n",
                         ofs, frame_width);
                     break;
                 }
@@ -302,7 +318,10 @@ static void vmd_decode(VmdVideoContext *s)
 
         case 2:
             for (i = 0; i < frame_height; i++) {
-                bytestream2_get_buffer(&gb, dp, frame_width);
+                if (pb_end -pb < frame_width)
+                    return;
+                memcpy(dp, pb, frame_width);
+                pb += frame_width;
                 dp += s->frame.linesize[0];
                 pp += s->prev_frame.linesize[0];
             }
@@ -312,20 +331,22 @@ static void vmd_decode(VmdVideoContext *s)
             for (i = 0; i < frame_height; i++) {
                 ofs = 0;
                 do {
-                    len = bytestream2_get_byte(&gb);
+                    if (pb_end - pb < 1)
+                        return;
+                    len = *pb++;
                     if (len & 0x80) {
                         len = (len & 0x7F) + 1;
-                        if (bytestream2_get_byte(&gb) == 0xFF)
-                            len = rle_unpack(gb.buffer, &dp[ofs],
-                                             len, bytestream2_get_bytes_left(&gb),
-                                             frame_width - ofs);
+                        if (pb_end - pb < 1)
+                            return;
+                        if (*pb++ == 0xFF)
+                            len = rle_unpack(pb, pb_end - pb, len, &dp[ofs], frame_width - ofs);
                         else {
-                            if (ofs + len > frame_width ||
-                                bytestream2_get_bytes_left(&gb) < len)
-                                return;
-                            bytestream2_get_buffer(&gb, &dp[ofs], len);
+                        if (pb_end - pb < len)
+                            return;
+                            memcpy(&dp[ofs], pb, len);
                         }
-                        bytestream2_skip(&gb, len);
+                        pb += len;
+                        ofs += len;
                     } else {
                         /* interframe pixel copy */
                         if (ofs + len + 1 > frame_width || !s->prev_frame.data[0])
@@ -335,7 +356,7 @@ static void vmd_decode(VmdVideoContext *s)
                     }
                 } while (ofs < frame_width);
                 if (ofs > frame_width) {
-                    av_log(s->avctx, AV_LOG_ERROR, "VMD video: offset > width (%d > %d)\n",
+                    av_log(s->avctx, AV_LOG_ERROR, "offset > width (%d > %d)\n",
                         ofs, frame_width);
                 }
                 dp += s->frame.linesize[0];
@@ -361,7 +382,7 @@ static av_cold int vmdvideo_decode_init(AVCodecContext *avctx)
 
     /* make sure the VMD header made it */
     if (s->avctx->extradata_size != VMD_HEADER_SIZE) {
-        av_log(s->avctx, AV_LOG_ERROR, "VMD video: expected extradata size of %d\n",
+        av_log(s->avctx, AV_LOG_ERROR, "expected extradata size of %d\n",
             VMD_HEADER_SIZE);
         return -1;
     }
@@ -403,8 +424,8 @@ static int vmdvideo_decode_frame(AVCodecContext *avctx,
         return buf_size;
 
     s->frame.reference = 3;
-    if (ff_get_buffer(avctx, &s->frame)) {
-        av_log(s->avctx, AV_LOG_ERROR, "VMD Video: get_buffer() failed\n");
+    if (avctx->get_buffer(avctx, &s->frame)) {
+        av_log(s->avctx, AV_LOG_ERROR, "get_buffer() failed\n");
         return -1;
     }
 
@@ -576,7 +597,7 @@ static int vmdaudio_decode_frame(AVCodecContext *avctx, void *data,
 
     /* get output buffer */
     s->frame.nb_samples = ((silent_chunks + audio_chunks) * avctx->block_align) / avctx->channels;
-    if ((ret = ff_get_buffer(avctx, &s->frame)) < 0) {
+    if ((ret = avctx->get_buffer(avctx, &s->frame)) < 0) {
         av_log(avctx, AV_LOG_ERROR, "get_buffer() failed\n");
         return ret;
     }
@@ -625,22 +646,22 @@ static int vmdaudio_decode_frame(AVCodecContext *avctx, void *data,
 AVCodec ff_vmdvideo_decoder = {
     .name           = "vmdvideo",
     .type           = AVMEDIA_TYPE_VIDEO,
-    .id             = CODEC_ID_VMDVIDEO,
+    .id             = AV_CODEC_ID_VMDVIDEO,
     .priv_data_size = sizeof(VmdVideoContext),
     .init           = vmdvideo_decode_init,
     .close          = vmdvideo_decode_end,
     .decode         = vmdvideo_decode_frame,
     .capabilities   = CODEC_CAP_DR1,
-    .long_name = NULL_IF_CONFIG_SMALL("Sierra VMD video"),
+    .long_name      = NULL_IF_CONFIG_SMALL("Sierra VMD video"),
 };
 
 AVCodec ff_vmdaudio_decoder = {
     .name           = "vmdaudio",
     .type           = AVMEDIA_TYPE_AUDIO,
-    .id             = CODEC_ID_VMDAUDIO,
+    .id             = AV_CODEC_ID_VMDAUDIO,
     .priv_data_size = sizeof(VmdAudioContext),
     .init           = vmdaudio_decode_init,
     .decode         = vmdaudio_decode_frame,
     .capabilities   = CODEC_CAP_DR1,
-    .long_name = NULL_IF_CONFIG_SMALL("Sierra VMD audio"),
+    .long_name      = NULL_IF_CONFIG_SMALL("Sierra VMD audio"),
 };
