@@ -20,6 +20,7 @@
  */
 #include "avformat.h"
 #include "internal.h"
+#include "libavutil/pixdesc.h"
 
 #define Y4M_MAGIC "YUV4MPEG2"
 #define Y4M_FRAME_MAGIC "FRAME"
@@ -60,6 +61,9 @@ static int yuv4_generate_header(AVFormatContext *s, char* buf)
     case PIX_FMT_GRAY8:
         colorspace = " Cmono";
         break;
+    case PIX_FMT_GRAY16:
+        colorspace = " Cmono16";
+        break;
     case PIX_FMT_YUV411P:
         colorspace = " C411 XYSCSS=411";
         break;
@@ -76,6 +80,33 @@ static int yuv4_generate_header(AVFormatContext *s, char* buf)
     case PIX_FMT_YUV444P:
         colorspace = " C444 XYSCSS=444";
         break;
+    case PIX_FMT_YUV420P9:
+        colorspace = " C420p9 XYSCSS=420P9";
+        break;
+    case PIX_FMT_YUV422P9:
+        colorspace = " C422p9 XYSCSS=422P9";
+        break;
+    case PIX_FMT_YUV444P9:
+        colorspace = " C444p9 XYSCSS=444P9";
+        break;
+    case PIX_FMT_YUV420P10:
+        colorspace = " C420p10 XYSCSS=420P10";
+        break;
+    case PIX_FMT_YUV422P10:
+        colorspace = " C422p10 XYSCSS=422P10";
+        break;
+    case PIX_FMT_YUV444P10:
+        colorspace = " C444p10 XYSCSS=444P10";
+        break;
+    case PIX_FMT_YUV420P16:
+        colorspace = " C420p16 XYSCSS=420P16";
+        break;
+    case PIX_FMT_YUV422P16:
+        colorspace = " C422p16 XYSCSS=422P16";
+        break;
+    case PIX_FMT_YUV444P16:
+        colorspace = " C444p16 XYSCSS=444P16";
+        break;
     }
 
     /* construct stream header, if this is the first frame */
@@ -90,7 +121,7 @@ static int yuv4_write_packet(AVFormatContext *s, AVPacket *pkt)
 {
     AVStream *st = s->streams[pkt->stream_index];
     AVIOContext *pb = s->pb;
-    AVPicture *picture;
+    AVPicture *picture, picture_tmp;
     int* first_pkt = s->priv_data;
     int width, height, h_chroma_shift, v_chroma_shift;
     int i;
@@ -98,7 +129,8 @@ static int yuv4_write_packet(AVFormatContext *s, AVPacket *pkt)
     char buf1[20];
     uint8_t *ptr, *ptr1, *ptr2;
 
-    picture = (AVPicture *)pkt->data;
+    memcpy(&picture_tmp, pkt->data, sizeof(AVPicture));
+    picture = &picture_tmp;
 
     /* for the first packet we have to output the header as well */
     if (*first_pkt) {
@@ -121,12 +153,39 @@ static int yuv4_write_packet(AVFormatContext *s, AVPacket *pkt)
     height = st->codec->height;
 
     ptr = picture->data[0];
+
+    switch (st->codec->pix_fmt) {
+    case PIX_FMT_GRAY8:
+    case PIX_FMT_YUV411P:
+    case PIX_FMT_YUV420P:
+    case PIX_FMT_YUV422P:
+    case PIX_FMT_YUV444P:
+        break;
+    case PIX_FMT_GRAY16:
+    case PIX_FMT_YUV420P9:
+    case PIX_FMT_YUV422P9:
+    case PIX_FMT_YUV444P9:
+    case PIX_FMT_YUV420P10:
+    case PIX_FMT_YUV422P10:
+    case PIX_FMT_YUV444P10:
+    case PIX_FMT_YUV420P16:
+    case PIX_FMT_YUV422P16:
+    case PIX_FMT_YUV444P16:
+        width *= 2;
+        break;
+    default:
+        av_log(s, AV_LOG_ERROR, "The pixel format '%s' is not supported.\n",
+               av_get_pix_fmt_name(st->codec->pix_fmt));
+        return AVERROR(EINVAL);
+    }
+
     for (i = 0; i < height; i++) {
         avio_write(pb, ptr, width);
         ptr += picture->linesize[0];
     }
 
-    if (st->codec->pix_fmt != PIX_FMT_GRAY8) {
+    if (st->codec->pix_fmt != PIX_FMT_GRAY8 &&
+        st->codec->pix_fmt != PIX_FMT_GRAY16) {
         // Adjust for smaller Cb and Cr planes
         avcodec_get_chroma_sub_sample(st->codec->pix_fmt, &h_chroma_shift,
                                       &v_chroma_shift);
@@ -144,6 +203,7 @@ static int yuv4_write_packet(AVFormatContext *s, AVPacket *pkt)
             ptr2 += picture->linesize[2];
         }
     }
+
     avio_flush(pb);
     return 0;
 }
@@ -155,20 +215,48 @@ static int yuv4_write_header(AVFormatContext *s)
     if (s->nb_streams != 1)
         return AVERROR(EIO);
 
-    if (s->streams[0]->codec->codec_id != CODEC_ID_RAWVIDEO) {
-        av_log(s, AV_LOG_ERROR, "ERROR: Only rawvideo supported.\n");
-        return AVERROR_INVALIDDATA;
+    if (s->streams[0]->codec->codec_id != AV_CODEC_ID_RAWVIDEO) {
+        av_log(s, AV_LOG_ERROR,
+               "A non-rawvideo stream was selected, but yuv4mpeg only handles rawvideo streams\n");
+        return AVERROR(EINVAL);
     }
 
-    if (s->streams[0]->codec->pix_fmt == PIX_FMT_YUV411P) {
-        av_log(s, AV_LOG_ERROR, "Warning: generating rarely used 4:1:1 YUV "
+    switch (s->streams[0]->codec->pix_fmt) {
+    case PIX_FMT_YUV411P:
+        av_log(s, AV_LOG_WARNING, "Warning: generating rarely used 4:1:1 YUV "
                "stream, some mjpegtools might not work.\n");
-    } else if ((s->streams[0]->codec->pix_fmt != PIX_FMT_YUV420P) &&
-               (s->streams[0]->codec->pix_fmt != PIX_FMT_YUV422P) &&
-               (s->streams[0]->codec->pix_fmt != PIX_FMT_GRAY8)   &&
-               (s->streams[0]->codec->pix_fmt != PIX_FMT_YUV444P)) {
-        av_log(s, AV_LOG_ERROR, "ERROR: yuv4mpeg only handles yuv444p, "
-               "yuv422p, yuv420p, yuv411p and gray pixel formats. "
+        break;
+    case PIX_FMT_GRAY8:
+    case PIX_FMT_GRAY16:
+    case PIX_FMT_YUV420P:
+    case PIX_FMT_YUV422P:
+    case PIX_FMT_YUV444P:
+        break;
+    case PIX_FMT_YUV420P9:
+    case PIX_FMT_YUV422P9:
+    case PIX_FMT_YUV444P9:
+    case PIX_FMT_YUV420P10:
+    case PIX_FMT_YUV422P10:
+    case PIX_FMT_YUV444P10:
+    case PIX_FMT_YUV420P16:
+    case PIX_FMT_YUV422P16:
+    case PIX_FMT_YUV444P16:
+        if (s->streams[0]->codec->strict_std_compliance >= FF_COMPLIANCE_NORMAL) {
+            av_log(s, AV_LOG_ERROR, "'%s' is not a official yuv4mpegpipe pixel format. "
+                   "Use '-strict -1' to encode to this pixel format.\n",
+                   av_get_pix_fmt_name(s->streams[0]->codec->pix_fmt));
+            return AVERROR(EINVAL);
+        }
+        av_log(s, AV_LOG_WARNING, "Warning: generating non standart YUV stream. "
+               "Mjpegtools will not work.\n");
+        break;
+    default:
+        av_log(s, AV_LOG_ERROR, "ERROR: yuv4mpeg can only handle "
+               "yuv444p, yuv422p, yuv420p, yuv411p and gray8 pixel formats. "
+               "And using 'strict -1' also yuv444p9, yuv422p9, yuv420p9, "
+               "yuv444p10, yuv422p10, yuv420p10, "
+               "yuv444p16, yuv422p16, yuv420p16 "
+               "and gray16 pixel formats. "
                "Use -pix_fmt to select one.\n");
         return AVERROR(EIO);
     }
@@ -179,11 +267,11 @@ static int yuv4_write_header(AVFormatContext *s)
 
 AVOutputFormat ff_yuv4mpegpipe_muxer = {
     .name              = "yuv4mpegpipe",
-    .long_name         = NULL_IF_CONFIG_SMALL("YUV4MPEG pipe format"),
+    .long_name         = NULL_IF_CONFIG_SMALL("YUV4MPEG pipe"),
     .extensions        = "y4m",
     .priv_data_size    = sizeof(int),
-    .audio_codec       = CODEC_ID_NONE,
-    .video_codec       = CODEC_ID_RAWVIDEO,
+    .audio_codec       = AV_CODEC_ID_NONE,
+    .video_codec       = AV_CODEC_ID_RAWVIDEO,
     .write_header      = yuv4_write_header,
     .write_packet      = yuv4_write_packet,
     .flags             = AVFMT_RAWPICTURE,
@@ -194,7 +282,7 @@ AVOutputFormat ff_yuv4mpegpipe_muxer = {
 #define MAX_YUV4_HEADER 80
 #define MAX_FRAME_HEADER 80
 
-static int yuv4_read_header(AVFormatContext *s, AVFormatParameters *ap)
+static int yuv4_read_header(AVFormatContext *s)
 {
     char header[MAX_YUV4_HEADER + 10];  // Include headroom for
                                         // the longest option
@@ -248,17 +336,40 @@ static int yuv4_read_header(AVFormatContext *s, AVFormatParameters *ap)
             } else if (strncmp("420paldv", tokstart, 8) == 0) {
                 pix_fmt = PIX_FMT_YUV420P;
                 chroma_sample_location = AVCHROMA_LOC_TOPLEFT;
-            } else if (strncmp("411", tokstart, 3) == 0)
+            } else if (strncmp("420p16", tokstart, 6) == 0) {
+                pix_fmt = PIX_FMT_YUV420P16;
+            } else if (strncmp("422p16", tokstart, 6) == 0) {
+                pix_fmt = PIX_FMT_YUV422P16;
+            } else if (strncmp("444p16", tokstart, 6) == 0) {
+                pix_fmt = PIX_FMT_YUV444P16;
+            } else if (strncmp("420p10", tokstart, 6) == 0) {
+                pix_fmt = PIX_FMT_YUV420P10;
+            } else if (strncmp("422p10", tokstart, 6) == 0) {
+                pix_fmt = PIX_FMT_YUV422P10;
+            } else if (strncmp("444p10", tokstart, 6) == 0) {
+                pix_fmt = PIX_FMT_YUV444P10;
+            } else if (strncmp("420p9", tokstart, 5) == 0) {
+                pix_fmt = PIX_FMT_YUV420P9;
+            } else if (strncmp("422p9", tokstart, 5) == 0) {
+                pix_fmt = PIX_FMT_YUV422P9;
+            } else if (strncmp("444p9", tokstart, 5) == 0) {
+                pix_fmt = PIX_FMT_YUV444P9;
+            } else if (strncmp("420", tokstart, 3) == 0) {
+                pix_fmt = PIX_FMT_YUV420P;
+                chroma_sample_location = AVCHROMA_LOC_CENTER;
+            } else if (strncmp("411", tokstart, 3) == 0) {
                 pix_fmt = PIX_FMT_YUV411P;
-            else if (strncmp("422", tokstart, 3) == 0)
+            } else if (strncmp("422", tokstart, 3) == 0) {
                 pix_fmt = PIX_FMT_YUV422P;
-            else if (strncmp("444alpha", tokstart, 8) == 0 ) {
+            } else if (strncmp("444alpha", tokstart, 8) == 0 ) {
                 av_log(s, AV_LOG_ERROR, "Cannot handle 4:4:4:4 "
                        "YUV4MPEG stream.\n");
                 return -1;
-            } else if (strncmp("444", tokstart, 3) == 0)
+            } else if (strncmp("444", tokstart, 3) == 0) {
                 pix_fmt = PIX_FMT_YUV444P;
-            else if (strncmp("mono", tokstart, 4) == 0) {
+            } else if (strncmp("mono16", tokstart, 6) == 0) {
+                pix_fmt = PIX_FMT_GRAY16;
+            } else if (strncmp("mono", tokstart, 4) == 0) {
                 pix_fmt = PIX_FMT_GRAY8;
             } else {
                 av_log(s, AV_LOG_ERROR, "YUV4MPEG stream contains an unknown "
@@ -312,6 +423,24 @@ static int yuv4_read_header(AVFormatContext *s, AVFormatParameters *ap)
                     alt_pix_fmt = PIX_FMT_YUV420P;
                 else if (strncmp("420PALDV", tokstart, 8) == 0)
                     alt_pix_fmt = PIX_FMT_YUV420P;
+                else if (strncmp("420P9", tokstart, 5) == 0)
+                    alt_pix_fmt = PIX_FMT_YUV420P9;
+                else if (strncmp("422P9", tokstart, 5) == 0)
+                    alt_pix_fmt = PIX_FMT_YUV422P9;
+                else if (strncmp("444P9", tokstart, 5) == 0)
+                    alt_pix_fmt = PIX_FMT_YUV444P9;
+                else if (strncmp("420P10", tokstart, 6) == 0)
+                    alt_pix_fmt = PIX_FMT_YUV420P10;
+                else if (strncmp("422P10", tokstart, 6) == 0)
+                    alt_pix_fmt = PIX_FMT_YUV422P10;
+                else if (strncmp("444P10", tokstart, 6) == 0)
+                    alt_pix_fmt = PIX_FMT_YUV444P10;
+                else if (strncmp("420P16", tokstart, 6) == 0)
+                    alt_pix_fmt = PIX_FMT_YUV420P16;
+                else if (strncmp("422P16", tokstart, 6) == 0)
+                    alt_pix_fmt = PIX_FMT_YUV422P16;
+                else if (strncmp("444P16", tokstart, 6) == 0)
+                    alt_pix_fmt = PIX_FMT_YUV444P16;
                 else if (strncmp("411", tokstart, 3) == 0)
                     alt_pix_fmt = PIX_FMT_YUV411P;
                 else if (strncmp("422", tokstart, 3) == 0)
@@ -357,7 +486,7 @@ static int yuv4_read_header(AVFormatContext *s, AVFormatParameters *ap)
     avpriv_set_pts_info(st, 64, rated, raten);
     st->codec->pix_fmt                = pix_fmt;
     st->codec->codec_type             = AVMEDIA_TYPE_VIDEO;
-    st->codec->codec_id               = CODEC_ID_RAWVIDEO;
+    st->codec->codec_id               = AV_CODEC_ID_RAWVIDEO;
     st->sample_aspect_ratio           = (AVRational){ aspectn, aspectd };
     st->codec->chroma_sample_location = chroma_sample_location;
 
@@ -368,7 +497,7 @@ static int yuv4_read_packet(AVFormatContext *s, AVPacket *pkt)
 {
     int i;
     char header[MAX_FRAME_HEADER+1];
-    int packet_size, width, height, ret;
+    int packet_size, width, height;
     AVStream *st = s->streams[0];
     struct frame_attributes *s1 = s->priv_data;
 
@@ -379,28 +508,20 @@ static int yuv4_read_packet(AVFormatContext *s, AVPacket *pkt)
             break;
         }
     }
-    if (s->pb->error)
-        return s->pb->error;
-    else if (s->pb->eof_reached)
-        return AVERROR_EOF;
-    else if (i == MAX_FRAME_HEADER)
-        return AVERROR_INVALIDDATA;
-
+    if (i == MAX_FRAME_HEADER)
+        return -1;
     if (strncmp(header, Y4M_FRAME_MAGIC, strlen(Y4M_FRAME_MAGIC)))
-        return AVERROR_INVALIDDATA;
+        return -1;
 
     width  = st->codec->width;
     height = st->codec->height;
 
     packet_size = avpicture_get_size(st->codec->pix_fmt, width, height);
     if (packet_size < 0)
-        return packet_size;
+        return -1;
 
-    ret = av_get_packet(s->pb, pkt, packet_size);
-    if (ret < 0)
-        return ret;
-    else if (ret != packet_size)
-        return s->pb->eof_reached ? AVERROR_EOF : AVERROR(EIO);
+    if (av_get_packet(s->pb, pkt, packet_size) != packet_size)
+        return AVERROR(EIO);
 
     if (st->codec->coded_frame) {
         st->codec->coded_frame->interlaced_frame = s1->interlaced_frame;
@@ -423,11 +544,11 @@ static int yuv4_probe(AVProbeData *pd)
 #if CONFIG_YUV4MPEGPIPE_DEMUXER
 AVInputFormat ff_yuv4mpegpipe_demuxer = {
     .name           = "yuv4mpegpipe",
-    .long_name      = NULL_IF_CONFIG_SMALL("YUV4MPEG pipe format"),
+    .long_name      = NULL_IF_CONFIG_SMALL("YUV4MPEG pipe"),
     .priv_data_size = sizeof(struct frame_attributes),
     .read_probe     = yuv4_probe,
     .read_header    = yuv4_read_header,
     .read_packet    = yuv4_read_packet,
-    .extensions     = "y4m"
+    .extensions     = "y4m",
 };
 #endif
